Package: ukbnmr
Type: Package
Title: Removal of Unwanted Technical Variation from UK Biobank NMR
        Metabolomics Biomarker Data
Version: 3.4
Authors@R: 
    person(given=c("Scott", "C"), family="Ritchie", email="sritchie73@gmail.com", 
           role=c("aut", "cre"), comment=c(ORCID="0000-0002-8454-9548"))
BugReports: https://github.com/sritchie73/ukbnmr/issues
Description: A suite of utilities for working with the UK Biobank 
    <https://www.ukbiobank.ac.uk/> Nuclear Magnetic Resonance spectroscopy (NMR) 
    metabolomics data <https://biobank.ndph.ox.ac.uk/showcase/label.cgi?id=220>. 
    Includes functions for extracting biomarkers from decoded UK Biobank field 
    data, removing unwanted technical variation from biomarker concentrations,
    computing an extended set of lipid, fatty acid, and cholesterol fractions, 
    and for re-deriving composite biomarkers and ratios after adjusting data 
    for unwanted biological variation. For further details on methods see
    Ritchie SC et al. Sci Data (2023) <doi:10.1038/s41597-023-01949-y>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 2.10)
Imports: data.table, bit64, lubridate, MASS
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Config/Needs/development: roxygen2
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-07 10:55:03 UTC; sr827
Author: Scott C Ritchie [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8454-9548>)
Maintainer: Scott C Ritchie <sritchie73@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-07 11:10:02 UTC
Built: R 4.5.2; ; 2026-07-07 13:45:01 UTC; unix
